@@ -1,9 +1,8 @@
-/* A Bison parser, made by GNU Bison 2.4.3.  */
+/* A Bison parser, made by GNU Bison 2.5.1.  */
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
+/* Bison interface for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-   2009, 2010 Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -45,7 +44,12 @@
      false = 262,
      null = 263,
      eof = 264,
-     error_initial = 265
+     error_initial = 265,
+     pinteger = 266,
+     ninteger = 267,
+     pzinteger = 268,
+     nzinteger = 269,
+     zero = 270
    };
 #endif
 
@@ -55,7 +59,7 @@
 typedef union YYSTYPE
 {
 
-/* Line 1685 of yacc.c  */
+/* Line 2072 of yacc.c  */
 #line 79 "json_parse_grammar.y"
 
     json_parse_u_obj	  uo;
@@ -64,8 +68,8 @@ typedef union YYSTYPE
 
 
 
-/* Line 1685 of yacc.c  */
-#line 69 "json_parse_grammar.tab.h"
+/* Line 2072 of yacc.c  */
+#line 73 "json_parse_grammar.tab.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
